(function (exports, React) {
  'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

  var CacheType;
  (function (CacheType) {
      CacheType["CREATE"] = "CREATE";
      CacheType["CREATED"] = "CREATED";
      CacheType["ACTIVE"] = "ACTIVE";
      CacheType["DESTROY"] = "DESTROY";
  })(CacheType || (CacheType = {}));
  var CacheType$1 = CacheType;

  const cacheReducer = (cacheStates, action) => {
      const { cacheId, reactElement, doms } = action.payload;
      switch (action.type) {
          case CacheType$1.CREATE:
              return Object.assign(Object.assign({}, cacheStates), { [cacheId]: {
                      cacheId,
                      reactElement,
                      doms: undefined,
                      status: CacheType$1.CREATE,
                      scrolls: {},
                  } });
          case CacheType$1.CREATED:
              return Object.assign(Object.assign({}, cacheStates), { [cacheId]: Object.assign(Object.assign({}, cacheStates[cacheId]), { doms: doms, status: CacheType$1.CREATED }) });
          case CacheType$1.DESTROY:
              return Object.assign(Object.assign({}, cacheStates), { [cacheId]: Object.assign(Object.assign({}, cacheStates[cacheId]), { status: CacheType$1.DESTROY }) });
          default:
              return cacheStates;
      }
  };

  const initialState = {};
  const CacheContext$1 = React__default["default"].createContext({
      cacheStates: {},
      dispatch: () => { },
      mount: () => { },
      handleScroll: () => { },
  });
  const KeepAliveProvider = props => {
      const [cacheStates, dispatch] = React.useReducer(cacheReducer, initialState);
      const mount = React.useCallback(({ cacheId, reactElement }) => {
          var _a;
          if (!cacheStates[cacheId]) {
              dispatch({
                  type: CacheType$1.CREATE,
                  payload: { cacheId, reactElement },
              });
          }
          else {
              if (cacheStates[cacheId].status === CacheType$1.DESTROY) {
                  (_a = cacheStates[cacheId].doms) === null || _a === void 0 ? void 0 : _a.forEach(dom => {
                      if (dom && dom.parentNode) {
                          dom.parentNode.removeChild(dom);
                      }
                  });
                  dispatch({
                      type: CacheType$1.CREATE,
                      payload: { cacheId, reactElement },
                  });
              }
          }
      }, [cacheStates]);
      const handleScroll = React.useCallback((cacheId, target) => {
          if (cacheStates[cacheId] && target) {
              const { scrolls } = cacheStates[cacheId];
              scrolls[target.baseURI] = target.scrollTop;
          }
      }, [cacheStates]);
      return (React__default["default"].createElement(CacheContext$1.Provider, { value: { cacheStates, dispatch, mount, handleScroll } },
          props.children,
          Object.values(cacheStates)
              .filter(cacheState => cacheState.status !== CacheType$1.DESTROY)
              .map(state => {
              const { cacheId, reactElement } = state;
              return (React__default["default"].createElement("div", { id: `cache-${cacheId}`, key: cacheId, ref: divDom => {
                      const cacheState = cacheStates[cacheId];
                      if (divDom &&
                          (!cacheState.doms || cacheState.status === CacheType$1.DESTROY)) {
                          const doms = Array.from(divDom.children);
                          dispatch({
                              type: CacheType$1.CREATED,
                              payload: { cacheId, reactElement, doms },
                          });
                      }
                  } }, reactElement));
          })));
  };

  const CacheContext = React__default["default"].createContext({
      cacheStates: {},
      dispatch: () => { },
      mount: () => { },
      handleScroll: () => { },
  });
  const withKeepAlive = (OldComponent, { cacheId = window.location.pathname, scroll }) => {
      return props => {
          const { cacheStates, dispatch, mount, handleScroll } = React.useContext(CacheContext);
          const divRef = React.useRef(null);
          const cacheDestroy = React.useCallback((cacheId) => () => {
              dispatch({ type: CacheType$1.DESTROY, payload: { cacheId } });
          }, [cacheId]);
          React.useEffect(() => {
              var _a;
              const onScroll = (event) => handleScroll(cacheId, event.target);
              if (divRef.current && scroll) {
                  divRef.current.addEventListener("scroll", onScroll, 
                  // ts需要制定target类型，事件的ts无法确定类型
                  true);
              }
              return (_a = divRef.current) === null || _a === void 0 ? void 0 : _a.removeEventListener("scroll", onScroll);
          }, [handleScroll]);
          React.useEffect(() => {
              const cacheState = cacheStates[cacheId];
              if (cacheState &&
                  cacheState.doms &&
                  cacheState.status !== CacheType$1.DESTROY) {
                  const doms = cacheState.doms;
                  doms.forEach(dom => {
                      if (divRef.current) {
                          divRef.current.appendChild(dom);
                          if (scroll && cacheState.scrolls[dom.baseURI]) {
                              dom.scrollTop = cacheState.scrolls[dom.baseURI];
                          }
                      }
                  });
              }
              else {
                  mount({
                      cacheId,
                      reactElement: React__default["default"].createElement(OldComponent, Object.assign({}, props, { cacheDestroy: cacheDestroy })),
                  });
              }
          }, [props, cacheStates, mount]);
          return React__default["default"].createElement("div", { id: `withKeepAlive${cacheId}`, ref: divRef });
      };
  };

  exports["default"] = KeepAliveProvider;
  exports.withKeepAlive = withKeepAlive;

  Object.defineProperty(exports, '__esModule', { value: true });

  return exports;

})({}, React);
//# sourceMappingURL=bundle.js.map
